[![FactSet](https://raw.githubusercontent.com/factset/enterprise-sdk/main/docs/images/factset-logo.svg)](https://www.factset.com)

# FactSet ETF client library for Python

[![API Version](https://img.shields.io/badge/api-v1.0.0-blue)]()
[![PyPi](https://img.shields.io/pypi/v/fds.sdk.FactSetETF)](https://pypi.org/project/fds.sdk.FactSetETF/)
[![Apache-2 license](https://img.shields.io/badge/license-Apache2-brightgreen.svg)](https://www.apache.org/licenses/LICENSE-2.0)

FactSet ETF data feeds provide complete and accurate security, fund and reference data across the universe of exchange-traded products. Data is sourced from ETF providers across the globe and includes more than 100 unique data points, resulting in comprehensive coverage to help you evaluate and construct ETFs, analyze potential trades, and perform fund research.<p> FactSet Reference and Analytics data uses FactSet's FCS, which categorizes exchange-traded products using a rules-based system that is derived from seven core classifications. This system evaluates the asset class, economic development level, and geographical region as described in each fund's prospectus and marketing materials. ETF exposure details are presented on successively granular levels-  category, then focus, and then niche.</p> <p>Moreover, FactSet ETF Reference data captures over 100 unique data points and provides market-specific data if you wish to solely focus on U.S., Canadian, Australian, New Zealand, Singapore, Hong Kong or European exchanges. All data points are grouped in one of two levels, either as a Fund or as a Listing. However, FactSet ETF Analytics data is primarily available for U.S.-domiciled ETFs. A subset of reference data items are provided for European-domiciled funds. For a full breakout of regional availability and history, see Formula Definitions and Calculations.</p>


This Python package is automatically generated by the [OpenAPI Generator](https://openapi-generator.tech) project:

- API version: 1.0.0
- SDK version: 1.0.12
- Build package: org.openapitools.codegen.languages.PythonClientCodegen

For more information, please visit [http://www.factset.com/api](http://www.factset.com/api)

## Requirements

* Python >= 3.7

## Installation

### Poetry

```shell
poetry add fds.sdk.utils fds.sdk.FactSetETF==1.0.12
```

### pip

```shell
pip install fds.sdk.utils fds.sdk.FactSetETF==1.0.12
```

## Usage

1. [Generate authentication credentials](../../../../README.md#authentication).
2. Setup Python environment.
   1. Install and activate python 3.7+. If you're using [pyenv](https://github.com/pyenv/pyenv):

      ```sh
      pyenv install 3.9.7
      pyenv shell 3.9.7
      ```

   2. (optional) [Install poetry](https://python-poetry.org/docs/#installation).
3. [Install dependencies](#installation).
4. Run the following:

> [!IMPORTANT]
> The parameter variables defined below are just examples and may potentially contain non valid values. Please replace them with valid values.

### Example Code

```python
from fds.sdk.utils.authentication import ConfidentialClient

import fds.sdk.FactSetETF
from fds.sdk.FactSetETF.api import data_items_api
from fds.sdk.FactSetETF.models import *
from dateutil.parser import parse as dateutil_parser
from pprint import pprint

# See configuration.py for a list of all supported configuration parameters.

# Examples for each supported authentication method are below,
# choose one that satisfies your use case.

# (Preferred) OAuth 2.0: FactSetOAuth2
# See https://github.com/FactSet/enterprise-sdk#oauth-20
# for information on how to create the app-config.json file
#
# The confidential client instance should be reused in production environments.
# See https://github.com/FactSet/enterprise-sdk-utils-python#authentication
# for more information on using the ConfidentialClient class
configuration = fds.sdk.FactSetETF.Configuration(
    fds_oauth_client=ConfidentialClient('/path/to/app-config.json')
)

# Basic authentication: FactSetApiKey
# See https://github.com/FactSet/enterprise-sdk#api-key
# for information how to create an API key
# configuration = fds.sdk.FactSetETF.Configuration(
#     username='USERNAME-SERIAL',
#     password='API-KEY'
# )

# Enter a context with an instance of the API client
with fds.sdk.FactSetETF.ApiClient(configuration) as api_client:
    # Create an instance of the API class
    api_instance = data_items_api.DataItemsApi(api_client)
    category = Category("BENCHMARK_DETAILS") # Category | The available categories that can be used to select collections of metrics for use within the ETF endpoints. |category|description| |---|---| |BENCHMARK_DETAILS|Details surrounding the underlying Benchmark Id and Segment Banchmark| |CLASSIFICATION|FactSet Fund Classification Codes and Names, across Asset Class, Broad  Geography, Fund Categories, Focus, Niche, and more.| |COSTS_FEES|Expenses and Fees such as capital gains, expense ratio, management fees, and more.| |COUNTERPARTY|Credit and Swap Counterparty details| |CREATE_REDEEM|Creation and Redemption Sizes| |DESCRIPTIVE|General Descriptive information such as name, objectives, issuer details, launch dates, website, and more.| |DISTRIBUTIONS|Dividend Dates, Dividend Treatmetns, Min/Max Cap Gains| |DOCUMENTATION|Details surrounding reporting information.| |GEARING|Leverage factors, inverse flags, and more.| |HEDGE|Hedging Information| |RISK|CIFSC Risk Ratings| |SERVICE_PROVIDERS|Distributors, issuers, and Advisor details| |STATUS|Actively Managed Flags| |STRATEGY|Segment Codes, selection criteria, strategy codes, weighting schemes, and lending details.| |STRUCTURE|ETF Type, backing codes, synthetic types, ucits compliance, legal structures, and more.| |TAX|Tax Types, distribution takes, K1 Flags, and more.|  (optional)

    try:
        # Available ETF metrics
        # example passing only required values which don't have defaults set
        # and optional values
        api_response = api_instance.get_etf_metrics(category=category)

        pprint(api_response)
    except fds.sdk.FactSetETF.ApiException as e:
        print("Exception when calling DataItemsApi->get_etf_metrics: %s\n" % e)

    # # Get response, http status code and response headers
    # try:
    #     # Available ETF metrics
    #     api_response, http_status_code, response_headers = api_instance.get_etf_metrics_with_http_info(category=category)


    #     pprint(api_response)
    #     pprint(http_status_code)
    #     pprint(response_headers)
    # except fds.sdk.FactSetETF.ApiException as e:
    #     print("Exception when calling DataItemsApi->get_etf_metrics: %s\n" % e)

    # # Get response asynchronous
    # try:
    #     # Available ETF metrics
    #     async_result = api_instance.get_etf_metrics_async(category=category)
    #     api_response = async_result.get()


    #     pprint(api_response)
    # except fds.sdk.FactSetETF.ApiException as e:
    #     print("Exception when calling DataItemsApi->get_etf_metrics: %s\n" % e)

    # # Get response, http status code and response headers asynchronous
    # try:
    #     # Available ETF metrics
    #     async_result = api_instance.get_etf_metrics_with_http_info_async(category=category)
    #     api_response, http_status_code, response_headers = async_result.get()


    #     pprint(api_response)
    #     pprint(http_status_code)
    #     pprint(response_headers)
    # except fds.sdk.FactSetETF.ApiException as e:
    #     print("Exception when calling DataItemsApi->get_etf_metrics: %s\n" % e)

```

### Using Pandas

To convert an API response to a Pandas DataFrame, it is necessary to transform it first to a dictionary.
```python
import pandas as pd

response_dict = api_response.to_dict()['data']

simple_json_response = pd.DataFrame(response_dict)
nested_json_response = pd.json_normalize(response_dict)
```

### Debugging

The SDK uses the standard library [`logging`](https://docs.python.org/3/library/logging.html#module-logging) module.

Setting `debug` to `True` on an instance of the `Configuration` class sets the log-level of related packages to `DEBUG`
and enables additional logging in Pythons [HTTP Client](https://docs.python.org/3/library/http.client.html).

**Note**: This prints out sensitive information (e.g. the full request and response). Use with care.

```python
import logging
import fds.sdk.FactSetETF

logging.basicConfig(level=logging.DEBUG)

configuration = fds.sdk.FactSetETF.Configuration(...)
configuration.debug = True
```

### Configure a Proxy

You can pass proxy settings to the Configuration class:

* `proxy`: The URL of the proxy to use.
* `proxy_headers`: a dictionary to pass additional headers to the proxy (e.g. `Proxy-Authorization`).

```python
import fds.sdk.FactSetETF

configuration = fds.sdk.FactSetETF.Configuration(
    # ...
    proxy="http://secret:password@localhost:5050",
    proxy_headers={
        "Custom-Proxy-Header": "Custom-Proxy-Header-Value"
    }
)
```

### Custom SSL Certificate

TLS/SSL certificate verification can be configured with the following Configuration parameters:

* `ssl_ca_cert`: a path to the certificate to use for verification in `PEM` format.
* `verify_ssl`: setting this to `False` disables the verification of certificates.
  Disabling the verification is not recommended, but it might be useful during
  local development or testing.

```python
import fds.sdk.FactSetETF

configuration = fds.sdk.FactSetETF.Configuration(
    # ...
    ssl_ca_cert='/path/to/ca.pem'
)
```

### Request Retries

In case the request retry behaviour should be customized, it is possible to pass a `urllib3.Retry` object to the `retry` property of the Configuration.

```python
from urllib3 import Retry
import fds.sdk.FactSetETF

configuration = fds.sdk.FactSetETF.Configuration(
    # ...
)

configuration.retries = Retry(total=3, status_forcelist=[500, 502, 503, 504])
```


## Documentation for API Endpoints

All URIs are relative to *https://api.factset.com/content*

Class | Method | HTTP request | Description
------------ | ------------- | ------------- | -------------
*DataItemsApi* | [**get_etf_metrics**](https://github.com/FactSet/enterprise-sdk/tree/main/code/python/FactSetETF/v1/docs/DataItemsApi.md#get_etf_metrics) | **GET** /factset-etf/v1/metrics | Available ETF metrics
*ReferenceApi* | [**get_etf_reference_data**](https://github.com/FactSet/enterprise-sdk/tree/main/code/python/FactSetETF/v1/docs/ReferenceApi.md#get_etf_reference_data) | **GET** /factset-etf/v1/reference | Return reference data for an ETF.
*ReferenceApi* | [**get_etf_reference_data_for_list**](https://github.com/FactSet/enterprise-sdk/tree/main/code/python/FactSetETF/v1/docs/ReferenceApi.md#get_etf_reference_data_for_list) | **POST** /factset-etf/v1/reference | Fetch Reference Data for a large list of ETF securities.


## Documentation For Models

 - [Categories](https://github.com/FactSet/enterprise-sdk/tree/main/code/python/FactSetETF/v1/docs/Categories.md)
 - [Category](https://github.com/FactSet/enterprise-sdk/tree/main/code/python/FactSetETF/v1/docs/Category.md)
 - [ErrorResponse](https://github.com/FactSet/enterprise-sdk/tree/main/code/python/FactSetETF/v1/docs/ErrorResponse.md)
 - [ErrorResponseSubErrors](https://github.com/FactSet/enterprise-sdk/tree/main/code/python/FactSetETF/v1/docs/ErrorResponseSubErrors.md)
 - [EtfReferenceData](https://github.com/FactSet/enterprise-sdk/tree/main/code/python/FactSetETF/v1/docs/EtfReferenceData.md)
 - [EtfReferenceDataRequest](https://github.com/FactSet/enterprise-sdk/tree/main/code/python/FactSetETF/v1/docs/EtfReferenceDataRequest.md)
 - [EtfReferenceDataResponse](https://github.com/FactSet/enterprise-sdk/tree/main/code/python/FactSetETF/v1/docs/EtfReferenceDataResponse.md)
 - [Ids](https://github.com/FactSet/enterprise-sdk/tree/main/code/python/FactSetETF/v1/docs/Ids.md)
 - [Metric](https://github.com/FactSet/enterprise-sdk/tree/main/code/python/FactSetETF/v1/docs/Metric.md)
 - [Metrics](https://github.com/FactSet/enterprise-sdk/tree/main/code/python/FactSetETF/v1/docs/Metrics.md)
 - [MetricsResponse](https://github.com/FactSet/enterprise-sdk/tree/main/code/python/FactSetETF/v1/docs/MetricsResponse.md)


## Documentation For Authorization


## FactSetApiKey

- **Type**: HTTP basic authentication


## FactSetOAuth2

- **Type**: OAuth
- **Flow**: application
- **Authorization URL**: 
- **Scopes**: N/A


## Notes for Large OpenAPI documents
If the OpenAPI document is large, imports in fds.sdk.FactSetETF.apis and fds.sdk.FactSetETF.models may fail with a
RecursionError indicating the maximum recursion limit has been exceeded. In that case, there are a couple of solutions:

Solution 1:
Use specific imports for apis and models like:
- `from fds.sdk.FactSetETF.api.default_api import DefaultApi`
- `from fds.sdk.FactSetETF.model.pet import Pet`

Solution 2:
Before importing the package, adjust the maximum recursion limit as shown below:
```
import sys
sys.setrecursionlimit(1500)
import fds.sdk.FactSetETF
from fds.sdk.FactSetETF.apis import *
from fds.sdk.FactSetETF.models import *
```

## Contributing

Please refer to the [contributing guide](../../../../CONTRIBUTING.md).

## Copyright

Copyright 2022 FactSet Research Systems Inc

Licensed under the Apache License, Version 2.0 (the "License");
you may not use this file except in compliance with the License.
You may obtain a copy of the License at

    http://www.apache.org/licenses/LICENSE-2.0

Unless required by applicable law or agreed to in writing, software
distributed under the License is distributed on an "AS IS" BASIS,
WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
See the License for the specific language governing permissions and
limitations under the License.

